from typing import Union, List

from janis_assistant.templates.slurm import SlurmSingularityTemplate


class PeterMacTemplate(SlurmSingularityTemplate):
    def __init__(
        self,
        execution_dir: str = None,
        container_dir="/config/binaries/singularity/containers_devel/janis/",
        queues: Union[str, List[str]] = "prod_med,prod",
        singularity_version="3.4.0",
        send_job_emails=False,
        catch_slurm_errors=True,
        singularity_build_instructions=None,
        max_cores=40,
        max_ram=256,
    ):
        """Peter Mac (login node) template

        Template to run Janis / Cromwell at the Peter MacCallum Cancer Centre (Rosalind)

        :param execution_dir: Execution directory
        :param queues: The queue to submit jobs to
        :param container_dir: [OPTIONAL] Override the directory singularity containers are stored in
        :param singularity_version: The version of Singularity to use on the cluster
        :param send_job_emails: Send Slurm job notifications using the provided email
        :param catch_slurm_errors: Fail the task if Slurm kills the job (eg: memory / time)
        :param singularity_build_instructions: Sensible default for PeterMac template
        :param max_cores: Override maximum number of cores (default: 32)
        :param max_ram: Override maximum ram (default 508 [GB])
        """

        singload = "module load singularity"
        if singularity_version:
            singload += "/" + str(singularity_version)

        joined_queued = ",".join(queues) if isinstance(queues, list) else str(queues)

        # Very cromwell specific at the moment, need to generalise this later
        if not singularity_build_instructions:
            singularity_build_instructions = f"sbatch -p {joined_queued} --wait \
    --wrap 'unset SINGULARITY_TMPDIR && docker_subbed=$(sed -e 's/[^A-Za-z0-9._-]/_/g' <<< ${{docker}}) \
    && image={container_dir}/$docker_subbed.sif && singularity pull $image docker://${{docker}}'"

        super().__init__(
            mail_program="sendmail -t",
            execution_dir=execution_dir,
            container_dir=container_dir,
            queues=joined_queued,
            send_job_emails=send_job_emails,
            catch_slurm_errors=catch_slurm_errors,
            build_instructions=singularity_build_instructions,
            singularity_load_instructions=singload,
            limit_resources=False,
            max_cores=max_cores,
            max_ram=max_ram,
        )
